@@ -18,6 +18,7 @@
 
 
 from bzrlib import (
+    debug,
     errors,
     generate_ids,
     inventory,
@@ -25,7 +26,19 @@ from bzrlib import (
     revision,
     serializer,
     )
-from bzrlib.plugins.fastimport import commands, helpers, processor
+from bzrlib.trace import (
+    mutter,
+    note,
+    warning,
+    )
+from fastimport import (
+    helpers,
+    processor,
+    )
+
+from bzrlib.plugins.fastimport.helpers import (
+    mode_to_kind,
+    )
 
 
 _serializer_handles_escaping = hasattr(serializer.Serializer,
@@ -33,13 +46,11 @@ _serializer_handles_escaping = hasattr(serializer.Serializer,
 
 
 def copy_inventory(inv):
-    # This currently breaks revision-id matching
-    #if hasattr(inv, "_get_mutable_inventory"):
-    #    # TODO: Make this a public API on inventory
-    #    return inv._get_mutable_inventory()
-
-    # TODO: Shallow copy - deep inventory copying is expensive
-    return inv.copy()
+    entries = inv.iter_entries_by_dir()
+    inv = inventory.Inventory(None, inv.revision_id)
+    for path, inv_entry in entries:
+        inv.add(inv_entry.copy())
+    return inv
 
 
 class GenericCommitHandler(processor.CommitHandler):
@@ -67,6 +78,27 @@ class GenericCommitHandler(processor.CommitHandler):
         # then a fresh file-id is required.
         self._paths_deleted_this_commit = set()
 
+    def mutter(self, msg, *args):
+        """Output a mutter but add context."""
+        msg = "%s (%s)" % (msg, self.command.id)
+        mutter(msg, *args)
+
+    def debug(self, msg, *args):
+        """Output a mutter if the appropriate -D option was given."""
+        if "fast-import" in debug.debug_flags:
+            msg = "%s (%s)" % (msg, self.command.id)
+            mutter(msg, *args)
+
+    def note(self, msg, *args):
+        """Output a note but add context."""
+        msg = "%s (%s)" % (msg, self.command.id)
+        note(msg, *args)
+
+    def warning(self, msg, *args):
+        """Output a warning but add context."""
+        msg = "%s (%s)" % (msg, self.command.id)
+        warning(msg, *args)
+
     def pre_process_files(self):
         """Prepare for committing."""
         self.revision_id = self.gen_revision_id()
@@ -76,11 +108,11 @@ class GenericCommitHandler(processor.CommitHandler):
         self.data_for_commit[inventory.ROOT_ID] = []
 
         # Track the heads and get the real parent list
-        parents = self.cache_mgr.track_heads(self.command)
+        parents = self.cache_mgr.reftracker.track_heads(self.command)
 
         # Convert the parent commit-ids to bzr revision-ids
         if parents:
-            self.parents = [self.cache_mgr.revision_ids[p]
+            self.parents = [self.cache_mgr.lookup_committish(p)
                 for p in parents]
         else:
             self.parents = []
@@ -201,8 +233,21 @@ class GenericCommitHandler(processor.CommitHandler):
         """Get a Bazaar file identifier for a path."""
         return self.bzr_file_id_and_new(path)[0]
 
-    def _format_name_email(self, name, email):
+    def _utf8_decode(self, field, value):
+        try:
+            return value.decode('utf_8')
+        except UnicodeDecodeError:
+            # The spec says fields are *typically* utf8 encoded
+            # but that isn't enforced by git-fast-export (at least)
+            self.warning("%s not in utf8 - replacing unknown "
+                "characters" % (field,))
+            return value.decode('utf_8', 'replace')
+
+    def _format_name_email(self, section, name, email):
         """Format name & email as a string."""
+        name = self._utf8_decode("%s name" % section, name)
+        email = self._utf8_decode("%s email" % section, email)
+
         if email:
             return "%s <%s>" % (name, email)
         else:
@@ -216,7 +261,7 @@ class GenericCommitHandler(processor.CommitHandler):
         committer = self.command.committer
         # Perhaps 'who' being the person running the import is ok? If so,
         # it might be a bit quicker and give slightly better compression?
-        who = self._format_name_email(committer[0], committer[1])
+        who = self._format_name_email("committer", committer[0], committer[1])
         timestamp = committer[2]
         return generate_ids.gen_revision_id(who, timestamp)
 
@@ -227,8 +272,14 @@ class GenericCommitHandler(processor.CommitHandler):
                     self.branch_ref)
         self._save_author_info(rev_props)
         committer = self.command.committer
-        who = self._format_name_email(committer[0], committer[1])
-        message = self.command.message
+        who = self._format_name_email("committer", committer[0], committer[1])
+        try:
+            message = self.command.message.decode("utf-8")
+
+        except UnicodeDecodeError:
+            self.warning(
+                "commit message not in utf8 - replacing unknown characters")
+            message = self.command.message.decode('utf-8', 'replace')
         if not _serializer_handles_escaping:
             # We need to assume the bad ol' days
             message = helpers.escape_commit_message(message)
@@ -262,9 +313,9 @@ class GenericCommitHandler(processor.CommitHandler):
             return
         if self.command.more_authors:
             authors = [author] + self.command.more_authors
-            author_ids = [self._format_name_email(a[0], a[1]) for a in authors]
+            author_ids = [self._format_name_email("author", a[0], a[1]) for a in authors]
         elif author != self.command.committer:
-            author_ids = [self._format_name_email(author[0], author[1])]
+            author_ids = [self._format_name_email("author", author[0], author[1])]
         else:
             return
         # If we reach here, there are authors worth storing
@@ -302,7 +353,7 @@ class GenericCommitHandler(processor.CommitHandler):
             # make sure the cache used by get_lines knows that
             self.data_for_commit[file_id] = ''
         elif kind == 'symlink':
-            ie.symlink_target = data.encode('utf8')
+            ie.symlink_target = data.decode('utf8')
             # There are no lines stored for a symlink so
             # make sure the cache used by get_lines knows that
             self.data_for_commit[file_id] = ''
@@ -311,7 +362,7 @@ class GenericCommitHandler(processor.CommitHandler):
                 % (kind, path))
             return
         # Record it
-        if file_id in inv:
+        if inv.has_id(file_id):
             old_ie = inv[file_id]
             if old_ie.kind == 'directory':
                 self.record_delete(path, old_ie)
@@ -353,7 +404,7 @@ class GenericCommitHandler(processor.CommitHandler):
 
         # It's possible that a file or symlink with that file-id
         # already exists. If it does, we need to delete it.
-        if dir_file_id in inv:
+        if inv.has_id(dir_file_id):
             self.record_delete(dirname, ie)
         self.record_new(dirname, ie)
         return basename, ie.file_id
@@ -424,10 +475,10 @@ class GenericCommitHandler(processor.CommitHandler):
                 content = self.rev_store.get_file_text(self.parents[0], file_id)
             self._modify_item(dest_path, kind, ie.executable, content, inv)
         elif kind == 'symlink':
-            self._modify_item(dest_path, kind, False, ie.symlink_target, inv)
+            self._modify_item(dest_path, kind, False, ie.symlink_target.encode("utf-8"), inv)
         else:
             self.warning("ignoring copy of %s %s - feature not yet supported",
-                kind, path)
+                kind, dest_path)
 
     def _rename_item(self, old_path, new_path, inv):
         existing = self._new_file_ids.get(old_path) or \
@@ -458,8 +509,11 @@ class GenericCommitHandler(processor.CommitHandler):
         self.data_for_commit[file_id] = ''.join(lines)
 
     def _delete_all_items(self, inv):
-        for name, root_item in inv.root.children.iteritems():
-            inv.remove_recursive_id(root_item.file_id)
+        if len(inv) == 0:
+            return
+        for path, ie in inv.iter_entries_by_dir():
+            if path != "":
+                self.record_delete(path, ie)
 
     def _warn_unless_in_merges(self, fileid, path):
         if len(self.parents) <= 1:
@@ -551,22 +605,23 @@ class InventoryCommitHandler(GenericCommitHandler):
         else:
             data = filecmd.data
         self.debug("modifying %s", filecmd.path)
-        self._modify_item(filecmd.path, filecmd.kind,
-            filecmd.is_executable, data, self.inventory)
+        (kind, is_executable) = mode_to_kind(filecmd.mode)
+        self._modify_item(filecmd.path.decode('utf8'), kind,
+            is_executable, data, self.inventory)
 
     def delete_handler(self, filecmd):
         self.debug("deleting %s", filecmd.path)
-        self._delete_item(filecmd.path, self.inventory)
+        self._delete_item(filecmd.path.decode('utf8'), self.inventory)
 
     def copy_handler(self, filecmd):
-        src_path = filecmd.src_path
-        dest_path = filecmd.dest_path
+        src_path = filecmd.src_path.decode('utf8')
+        dest_path = filecmd.dest_path.decode('utf8')
         self.debug("copying %s to %s", src_path, dest_path)
         self._copy_item(src_path, dest_path, self.inventory)
 
     def rename_handler(self, filecmd):
-        old_path = filecmd.old_path
-        new_path = filecmd.new_path
+        old_path = filecmd.old_path.decode('utf8')
+        new_path = filecmd.new_path.decode('utf8')
         self.debug("renaming %s to %s", old_path, new_path)
         self._rename_item(old_path, new_path, self.inventory)
 
@@ -650,7 +705,7 @@ class InventoryDeltaCommitHandler(GenericCommitHandler):
             if len(ie.children) == 0:
                 result.append((dir, file_id))
                 if self.verbose:
-                    self.note("pruning empty directory %s" % (dir,))                
+                    self.note("pruning empty directory %s" % (dir,))
         return result
 
     def _get_proposed_inventory(self, delta):
@@ -821,41 +876,36 @@ class InventoryDeltaCommitHandler(GenericCommitHandler):
         self.record_new(new_path, ie)
 
     def modify_handler(self, filecmd):
+        (kind, executable) = mode_to_kind(filecmd.mode)
         if filecmd.dataref is not None:
-            if filecmd.kind == commands.DIRECTORY_KIND:
+            if kind == "directory":
                 data = None
-            elif filecmd.kind == commands.TREE_REFERENCE_KIND:
+            elif kind == "tree-reference":
                 data = filecmd.dataref
             else:
                 data = self.cache_mgr.fetch_blob(filecmd.dataref)
         else:
             data = filecmd.data
         self.debug("modifying %s", filecmd.path)
-        self._modify_item(filecmd.path, filecmd.kind,
-            filecmd.is_executable, data, self.basis_inventory)
+        self._modify_item(filecmd.path.decode('utf8'), kind,
+            executable, data, self.basis_inventory)
 
     def delete_handler(self, filecmd):
         self.debug("deleting %s", filecmd.path)
-        self._delete_item(filecmd.path, self.basis_inventory)
+        self._delete_item(filecmd.path.decode('utf8'), self.basis_inventory)
 
     def copy_handler(self, filecmd):
-        src_path = filecmd.src_path
-        dest_path = filecmd.dest_path
+        src_path = filecmd.src_path.decode("utf8")
+        dest_path = filecmd.dest_path.decode("utf8")
         self.debug("copying %s to %s", src_path, dest_path)
         self._copy_item(src_path, dest_path, self.basis_inventory)
 
     def rename_handler(self, filecmd):
-        old_path = filecmd.old_path
-        new_path = filecmd.new_path
+        old_path = filecmd.old_path.decode("utf8")
+        new_path = filecmd.new_path.decode("utf8")
         self.debug("renaming %s to %s", old_path, new_path)
         self._rename_item(old_path, new_path, self.basis_inventory)
 
     def deleteall_handler(self, filecmd):
         self.debug("deleting all files (and also all directories)")
-        # I'm not 100% sure this will work in the delta case.
-        # But clearing out the basis inventory so that everything
-        # is added sounds ok in theory ...
-        # We grab a copy as the basis is likely to be cached and
-        # we don't want to destroy the cached version
-        self.basis_inventory = copy_inventory(self.basis_inventory)
         self._delete_all_items(self.basis_inventory)
