@@ -64,8 +64,13 @@ class _TreeShim(object):
         return self._basis_inv.root.file_id
 
     def get_file_with_stat(self, file_id, path=None):
+        content = self.get_file_text(file_id, path)
+        sio = cStringIO.StringIO(content)
+        return sio, None
+
+    def get_file_text(self, file_id, path=None):
         try:
-            content = self._content_provider(file_id)
+            return self._content_provider(file_id)
         except KeyError:
             # The content wasn't shown as 'new'. Just validate this fact
             assert file_id not in self._new_info_by_id
@@ -73,9 +78,7 @@ class _TreeShim(object):
             old_text_key = (file_id, old_ie.revision)
             stream = self._repo.texts.get_record_stream([old_text_key],
                                                         'unordered', True)
-            content = stream.next().get_bytes_as('fulltext')
-        sio = cStringIO.StringIO(content)
-        return sio, None
+            return stream.next().get_bytes_as('fulltext')
 
     def get_symlink_target(self, file_id):
         if file_id in self._new_info_by_id:
@@ -268,7 +271,7 @@ class AbstractRevisionStore(object):
             parent_candidate_entries.keys())
         heads = []
         for inv in self._rev_parent_invs:
-            if ie.file_id in inv:
+            if inv.has_id(ie.file_id):
                 old_rev = inv[ie.file_id].revision
                 if old_rev in head_set:
                     rev_id = inv[ie.file_id].revision
@@ -366,12 +369,13 @@ class AbstractRevisionStore(object):
             timezone=rev.timezone, committer=rev.committer,
             revprops=rev.properties, revision_id=rev.revision_id)
         if self._graph is None and self._use_known_graph:
-            if (getattr(_mod_graph, 'GraphThunkIdsToKeys', None) is None
-                or getattr(_mod_graph.KnownGraph, 'add_node', None) is None):
-                self._use_known_graph = False
+            if (getattr(_mod_graph, 'GraphThunkIdsToKeys', None) and
+                getattr(_mod_graph.GraphThunkIdsToKeys, "add_node", None) and
+                getattr(self.repo, "get_known_graph_ancestry", None)):
+                self._graph = self.repo.get_known_graph_ancestry(
+                    rev.parent_ids)
             else:
-                self._graph = self.repo.revisions.get_known_graph_ancestry(
-                    [(r,) for r in rev.parent_ids])
+                self._use_known_graph = False
         if self._graph is not None:
             orig_heads = builder._heads
             def thunked_heads(file_id, revision_ids):
@@ -381,8 +385,7 @@ class AbstractRevisionStore(object):
                 if len(revision_ids) < 2:
                     res = set(revision_ids)
                 else:
-                    res = set([h[0] for h in
-                              self._graph.heads([(r,) for r in revision_ids])])
+                    res = set(self._graph.heads(revision_ids))
                 # if old_res != res:
                 #     import pdb; pdb.set_trace()
                 return res
@@ -413,12 +416,11 @@ class AbstractRevisionStore(object):
             builder.new_inventory, builder._config)
         if self._graph is not None:
             # TODO: Use StaticTuple and .intern() for these things
-            self._graph.add_node((builder._new_revision_id,),
-                                 [(p,) for p in rev.parent_ids])
+            self._graph.add_node(builder._new_revision_id, rev.parent_ids)
 
         if signature is not None:
             raise AssertionError('signatures not guaranteed yet')
-            self.repo.add_signature_text(rev_id, signature)
+            self.repo.add_signature_text(rev.revision_id, signature)
         # self._add_revision(rev, inv)
         return builder.revision_tree().inventory
 
